#!/usr/bin/perl -w
#
# usage: -d Daily, -w Weekly, -M Monthly
#		 -r backup.run
#		 -S cartset

use strict;

use Getopt::Std;

my $clnbindir = "/usr/local/afbackup/client/bin";
my $srvbindir = "/usr/local/afbackup/server/bin";

my $clnlibdir = "/usr/local/afbackup/client/lib";
my $srvlibdir = "/usr/local/afbackup/server/lib";

my $clnvardir = "/var/log/afbackup/client";
my $srvvardir = "/var/log/afbackup/server";
my $schvardir = "/var/log/afbackup/scheduler";

my $afclient = "$clnbindir/afclient";
#my $afclient = "echo";
my $reuseTapes = "$clnbindir/update_indexes";

my $config_file = "$srvlibdir/dwm.schedule";
my $status_file = "$schvardir/dwm.next";

my %config;
my %status;
my @run_list;

my %opt;
getopts( "DWMr:S:" , \%opt );

my $cartset;
my $server;
my $serverport;
my $run;
my $mailto;
my $debug;

my $M1 ;

&read_status;
&read_config;

if ( $opt{D} ) {
	$run = $config{ "runDaily" };
	$cartset = $config { "D".$status{ "nextDaily" } };
	
	&reuse_cartset ( "D", $status{ "reuseDaily" } ) if ( $status{ "reuseDaily" } );
	
} elsif ( $opt{W} ) {
	$run = $config{ "runWeekly" };
	$cartset = $config { "W".$status{ "nextWeekly" } };
	
	$status{ "nextDaily" } ++ ;
	$status{ "reuseDaily" } = $status{ "nextDaily" };

	&reuse_cartset ( "W" ) if ( $status{ "reuseWeekly" } );
	

} elsif ( $opt{M} ) {
	$run = $config{ "runMonthly" };
	$cartset = $config { "M".$status{ 'nextMonthly'} };

	$status{ "nextMonthly" } ++ ;
	$status{ "nextMonthly" } = 1 if ( $status{ "nextMonthly" } > 3 );
	$status{ "nextWeekly" } = 	$status{ "nextMonthly" } ;
	$status{ "reuseWeekly" } = $status{ "nextWeekly" };
	$status{ "nextDaily" } = 1 ;
	$status{ "reuseDaily" } = $status{ "nextDaily" };

} else {
	$run = $opt{r};
	$cartset = $opt{S} ;
}


&read_runlist;

print "Schedule   : ".$srvlibdir."/".$run."\n";
print "running 	  : ".@run_list." jobs on server: $server:$serverport\n";
print "to CartSet : ".$cartset."\n\n";
#my $M1 = "M".$status{ 'nextMonthly'} ;
#print "to CartSet : ".$config{ $M1 }."\n\n";

&run_backup;

&write_status;
&write_config;

exit 0;


sub run_backup {
	foreach my $job (@run_list) {
		my ($clhost, $clconf, $full_incr, $level) = split(' ', $job);
		print "run : ".$full_incr." on host ".$clhost."\n";
	
		my $xstart = ( $full_incr eq "full") ? "full_backup " : "incr_backup -Q$level";
		my $param1 =   "-h".$clhost;
		my $param3 =   "$xstart -h$server -P$serverport -c$clconf -S$cartset" ;
		
		#`echo $clhost $mailto $debug`;
		#`echo "Start $clhost" | mailx -s "Run start" $mailto"` ; 

		system ($afclient, $param1, "-X", $param3);						
	}
}

sub reuse_cartset {
	my ($r, $c) = @_;
	
	print "reuse : $r , ".$status{ "reuseDaily" }.$status{ "reuseWeekly" }."\n";
	
	if ( $r eq "D" ) { $status {"reuseDaily"} = "" }
	else  { $status {"reuseWeekly"} = "" };	

	# remove cartset from server/precioustapes
	# fr alle clients :system ($update_indexes, $config{ $set } );
	#foreach $job (@run_list) {
	#	($clhost, $clconf, $dummy, $full_incr, $level) = split(' ', $job);
	#	print "run : ".$full_incr." on host ".$clhost."\n";
	#
	#	my $xstart = ( $full_incr eq "full") ? "full_backup " : "incr_backup -Q$level";
	#	my $param1 =   "-h".$clhost;
	#	my $param3 =   "$xstart -h$server -P$serverport -c$clconf -S$cartset" ;
	#	
	#	#`echo $clhost $mailto $debug`;
	#	#`echo "Start $clhost" | mailx -s "Run start" $mailto"` ; 
	#
	#	system ($afclient, $param1, "-X", $param3);						
}

sub read_runlist {
	open (RUN, $srvlibdir."/".$run) or die "Can't open file: ".$srvlibdir."/".$run;

	my $jobs_flag = 0;
	foreach my $line (<RUN>) {
		if ($jobs_flag == 0) { 
			$jobs_flag++ if ( $line =~ /jobs/ );	
			my ($key, $value) = split("=", $line);
			if ($value) {
				chomp($value);
				if 	($key eq "server") 	{ $server = $value }
				elsif	($key eq "serverport") 	{ $serverport = $value }  
				elsif	($key eq "mailto") 	{ $mailto = $value }  
				elsif	($key eq "debug") 	{ $debug = $value }
			}
		} else {  
			push (@run_list, $line);
		}
	} 
	close (RUN);
}

sub read_status {

	open STATUS, $status_file or die "can't open $status_file";
	foreach my $line (<STATUS>) {
		my ($key, $value) = split("=", $line);
		$status{"$key"} = $value;
		chomp $status{"$key"};
	}
	close STATUS;
}

sub read_config {

	open CONFIG, $config_file or die "can't open $config_file";
	foreach my $line (<CONFIG>) {
		my ($key, $value) = split("=", $line);
		$config{"$key"} = $value;
		chomp $config{"$key"};
	}
	close CONFIG;
}

sub write_config {
	open CONFIG, "> $config_file" or die "can't open $config_file for writing";
	print CONFIG "D1=".$config{ "D1" }."\n";
	print CONFIG "D2=".$config{ "D2" }."\n";
	print CONFIG "D3=".$config{ "D3" }."\n";
	print CONFIG "D4=".$config{ "D4" }."\n";
	print CONFIG "D5=".$config{ "D5" }."\n";
	print CONFIG "W1=".$config{ "W1" }."\n";
	print CONFIG "W2=".$config{ "W2" }."\n";
	print CONFIG "W3=".$config{ "W3" }."\n";
	print CONFIG "M1=".$config{ "M2" }."\n";
	print CONFIG "M2=".$config{ "M2" }."\n";
	print CONFIG "M3=".$config{ "M3" }."\n";
	print CONFIG "runDaily=".$config{ "runDaily" }."\n";
	print CONFIG "runWeekly=".$config{ "runWeekly"}."\n";
	print CONFIG "runMonthly=".$config{"runMonthly"}."\n";
	close CONFIG;
}

sub write_status {
	open STATUS, "> $status_file" or die "can't open $status_file for writing";
	print STATUS "nextDaily=".$status{ "nextDaily" }."\n";
	print STATUS "nextWeekly=".$status{ "nextWeekly" }."\n";
	print STATUS "nextMonthly=".$status{ "nextMonthly" }."\n";
	print STATUS "reuseDaily=".$status{ "reuseDaily" }."\n";
	print STATUS "reuseWeekly=".$status{ "reuseWeekly" }."\n";
	close STATUS;
}